@@ -1,17 +1,19 @@
 # This file is part of versuchung.
-# 
+#
 # versuchung is free software: you can redistribute it and/or modify it under the
 # terms of the GNU General Public License as published by the Free Software
 # Foundation, either version 3 of the License, or (at your option) any later
 # version.
-# 
+#
 # versuchung is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 # PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License along with
 # versuchung.  If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import print_function
+
 from versuchung.types import Type, InputParameter, OutputParameter
 import logging
 import sqlite3
@@ -91,7 +93,7 @@ class Database_MySQL(InputParameter, OutputParameter, Type, Database_Abstract):
         directory = self.tmp_directory.new_directory(self.name)
         path = os.path.join(directory.path, "my.cnf")
         logging.debug("MYSQL_HOME=%s", path)
-        with os.fdopen(os.open(path, os.O_WRONLY | os.O_CREAT, 0600), 'w') as handle:
+        with os.fdopen(os.open(path, os.O_WRONLY | os.O_CREAT, 0o600), 'w') as handle:
             handle.write("""[client]
 host=%s
 user=%s
@@ -135,7 +137,7 @@ database=%s
                          self.dynamic_experiment.experiment_identifier,
                          str(self.dynamic_experiment.metadata))
 
-    @property 
+    @property
     def handle(self):
         """:return: handle -- MySQLdb database handle"""
         assert self.__database_connection
@@ -153,7 +155,7 @@ database=%s
         self.__database_connection.commit()
         return c
 
-    def create_table(self, name, fields = [("key", "text"), ("value", "text")], 
+    def create_table(self, name, fields = [("key", "text"), ("value", "text")],
                      keys = None, conflict_strategy = None):
         """Creates a new table in the database. ``name`` is the name
         of newly created table. The ``fields`` are a list of
@@ -261,7 +263,7 @@ class Database_SQLite(InputParameter, OutputParameter, Type, Database_Abstract):
         """:return: string -- path to the sqlite database file"""
         return os.path.join(self.base_directory, self.__database_path)
 
-    @property 
+    @property
     def handle(self):
         """:return: handle -- sqlite3 database handle"""
         assert self.__database_connection
@@ -278,7 +280,7 @@ class Database_SQLite(InputParameter, OutputParameter, Type, Database_Abstract):
 
 
 
-    def create_table(self, name, fields = [("key", "text"), ("value", "text")], 
+    def create_table(self, name, fields = [("key", "text"), ("value", "text")],
                      keys = None, conflict_strategy = "REPLACE"):
         """Creates a new table in the database. ``name`` is the name
         of newly created table. The ``fields`` are a list of
@@ -322,17 +324,17 @@ class Table(InputParameter, OutputParameter, Type):
 
     A versuchung table is a table that is aware of experiments. It
     stores for each dataset the experiment it originates from. The
-    field list consits either of plain strings, then the column type
+    field list consists either of plain strings, then the column type
     is text. If it's a tuple the first entry is the name and the second its type::
 
     >>> [("foo", "integer"), "barfoo"]
-   
+
     This will result in two columns, one with type integer and one
     with type text. If a db is given this one is used instead of a
     default sqlite database named ``sqlite3.db``
 
     To make a set of field the index keys (UNIQUE), give it as a list
-    of string as keys argument. The conflict_stragety gives the SQL
+    of string as keys argument. The conflict_strategy gives the SQL
     strategy what to do on conflict. If you want to merge databases from
     multiple experiments without triggering a conflict if the given key
     set is equal (i.e., if you want the same values in the columns given
@@ -366,7 +368,7 @@ class Table(InputParameter, OutputParameter, Type):
         return real_fields
 
     def before_experiment_run(self, parameter_type):
-        # Add database object as an 
+        # Add database object as an
         self.subobjects["database"] = self.__db
         Type.before_experiment_run(self, parameter_type)
 
@@ -483,7 +485,7 @@ class TableDict(Table, dict):
 class Database_SQlite_Merger:
     def log(self, msg, *args):
         if self.logging:
-            print "merger: " + (msg % args)
+            print("merger: " + (msg % args))
 
     def __init__(self, target_path, source_paths = [], logging = True):
         self.target_path = target_path
@@ -535,7 +537,7 @@ class Database_SQlite_Merger:
         cur = self.target.cursor()
 
         TableDictrows = set()
-        
+
         for name in self.tables:
             rows = set()
             headers = None
@@ -555,7 +557,7 @@ class Database_SQlite_Merger:
 
             if headers == ["experiment", "key", "value"]:
                 TableDictrows.update(rows)
-        
+
         cur.execute("CREATE TABLE IF NOT EXISTS TableDict (experiment text, key text, value text,"
                     "UNIQUE (key) ON CONFLICT REPLACE)")
         cur.executemany("INSERT INTO TableDict (experiment, key, value) values(?,?,?)",
@@ -569,15 +571,14 @@ class Database_SQlite_Merger:
         self.collect_and_create_tables(drop = not update)
         self.collect_data()
         self.target.close()
-        
+
 
 if __name__ == '__main__':
     import sys
     if len(sys.argv) < 2:
-        print sys.argv[0] + " <target-database-file> [<source-db1> <source-db2> ...]"
-        print " -- merges different versuchung sqlite databases into a single one"
+        print(sys.argv[0] + " <target-database-file> [<source-db1> <source-db2> ...]")
+        print(" -- merges different versuchung sqlite databases into a single one")
         sys.exit(-1)
 
     merger = Database_SQlite_Merger(sys.argv[1], sys.argv[2:])
     merger.merge()
-
