@@ -1,22 +1,22 @@
 # This file is part of versuchung.
-# 
+#
 # versuchung is free software: you can redistribute it and/or modify it under the
 # terms of the GNU General Public License as published by the Free Software
 # Foundation, either version 3 of the License, or (at your option) any later
 # version.
-# 
+#
 # versuchung is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 # PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License along with
 # versuchung.  If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import print_function
 
 from optparse import OptionParser
 import datetime
 import logging
-import pprint
 from versuchung.types import InputParameter, OutputParameter, Type
 from versuchung.files import Directory
 from versuchung.tools import JavascriptStyleDictAccess, setup_logging
@@ -24,6 +24,7 @@ import sys
 import os.path
 import glob
 import hashlib
+import json
 import shutil
 import copy
 import tempfile
@@ -110,8 +111,8 @@ class Experiment(Type, InputParameter):
     def static_experiment(self, value):
         pass
 
-    def __init__(self, default_experiment_instance = None):
-        """The constructor of an experiment just filles in the
+    def __init__(self, default_experiment_instance = None, title=None,inputs=None,outputs=None):
+        """The constructor of an experiment just fills in the
         necessary attributes but has *no* sideeffects on the outside
         world.
 
@@ -124,12 +125,40 @@ class Experiment(Type, InputParameter):
         """
         Type.__init__(self)
         InputParameter.__init__(self)
-
-        self.title = self.__class__.__name__
+        self.title = title or self.__class__.__name__
         self.static_experiment = self
+        if inputs is not None:
+            self.__class__.inputs = inputs
+        if outputs is not None:
+            self.__class__.outputs = outputs
+        if default_experiment_instance and not os.path.exists(default_experiment_instance):
+            default_experiment_instance = None
+        self.__reinit__(default_experiment_instance)
 
-        self.__experiment_instance = default_experiment_instance
-        self.__metadata = None
+    def __reinit__(self, experiment_path):
+        self.__experiment_instance = experiment_path
+
+        if experiment_path:
+            if "/" in experiment_path:
+                self.__experiment_instance = os.path.basename(experiment_path)
+
+            self.base_directory = os.path.join(os.curdir, experiment_path)
+            self.base_directory = os.path.realpath(self.base_directory)
+            assert os.path.exists(self.base_directory)
+
+            try:
+                with open(os.path.join(experiment_path, "metadata"), "r") as fd:
+                    metadata = json.load(fd)
+            except json.JSONDecodeError:
+                if os.getenv("VERSUCHUNG_METADATA_EVAL"):
+                    with open(os.path.join(experiment_path, "metadata"), "r") as fd:
+                        metadata = eval(fd.read())
+                else:
+                    raise RuntimeError("metadata is invalid JSON. Set VERSUCHUNG_METADATA_EVAL=1 to load metadata exported via pprint") from None
+            self.__metadata = metadata
+        else:
+            self.base_directory = None
+            self.__metadata = None
 
         # Copy input and output objects
         self.inputs = JavascriptStyleDictAccess(copy.deepcopy(self.__class__.inputs))
@@ -137,11 +166,7 @@ class Experiment(Type, InputParameter):
         self.outputs = JavascriptStyleDictAccess(copy.deepcopy(self.__class__.outputs))
         self.o = self.outputs
 
-        if default_experiment_instance != None:
-            self.base_directory = os.path.join(os.curdir, self.__experiment_instance)
-            self.base_directory = os.path.realpath(self.base_directory)
-        else:
-            self.base_directory = os.path.realpath(os.curdir)
+        self.subobjects.clear()
 
         # Sanity checking for input parameters.
         for (name, inp) in self.inputs.items():
@@ -149,39 +174,54 @@ class Experiment(Type, InputParameter):
             if type(inp) == LambdaType:
                 continue
             if not isinstance(inp, InputParameter):
-                print "%s cannot be used as an input parameter" % name
+                print("%s cannot be used as an input parameter" % name)
                 sys.exit(-1)
             self.subobjects[name] = inp
 
+
         for (name, outp) in self.outputs.items():
             if not isinstance(outp, OutputParameter):
-                print "%s cannot be used as an output parameter" % name
+                print("%s cannot be used as an output parameter" % name)
                 sys.exit(-1)
             self.subobjects[name] = outp
 
+
+        # Reinit Children Attributes
+        if experiment_path:
+            for (name, inp) in self.inputs.items():
+                if hasattr(inp, "__reinit__"):
+                    try:
+                        inp.__reinit__(metadata[name])
+                    except:
+                        logging.debug('Cannot reinit field %s. Setting it to None', name)
+                        self.inputs[name] = None
+                else:
+                    # We cannot reinit this input from metadata. Therefore it is better to clear it.
+                    logging.debug('Cannot reinit field %s. Setting it to None', name)
+                    self.inputs[name] = None
+
+
+
     def __setup_parser(self):
         self.__parser = OptionParser("%prog <options>")
         self.__parser.add_option('-d', '--base-dir', dest='base_dir', action='store',
                                  help="Directory which is used for storing the experiment data",
                                  default = ".")
-        self.__parser.add_option('-l', '--list', dest='do_list', action='store_true',
-                                 help="list all experiment results")
+        self.__parser.add_option('--dummy', dest='dummy_result', action='store_true',
+                                 help="Use dummy result directory",
+                                 default=False)
         self.__parser.add_option('-s', '--symlink', dest='do_symlink', action='store_true',
                                  help="symlink the result dir (as newest)")
-        self.__parser.add_option('-v', '--verbose', dest='verbose', action='count',
+        self.__parser.add_option('-v', '--verbose', dest='verbose', action='count', default=0,
                                  help="increase verbosity (specify multiple times for more)")
+        self.__parser.add_option('--title', dest='title',
+                                 help="custom title of the experiment (default: Experiment class-name)")
 
         for (name, inp) in self.inputs.items():
             if type(inp) == LambdaType:
                 continue
             inp.inp_setup_cmdline_parser(self.__parser)
 
-    def __setup_tmp_directory(self):
-        """Creat temporary directory and assign it to every input and
-        output directories tmp_directory slots"""
-        # Create temp directory
-        self.tmp_directory = Directory(tempfile.mkdtemp())
-        self.subobjects["tmp_directory"] = self.tmp_directory
 
     def execute(self, args = [], **kwargs):
         """Calling this method will execute the experiment
@@ -204,25 +244,24 @@ class Experiment(Type, InputParameter):
         >>> experiment.execute(["--input_parameter", "foo"])
         >>> experiment.execute(input_parameter="foo")
         """
+        self.execute_setup(args, **kwargs)
+        self.execute_run()
+        return self.execute_teardown()
+
+    def execute_setup(self, args=[], **kwargs):
         self.dynamic_experiment = self
+        self.startup_directory = os.path.abspath(os.curdir)
+
         self.subobjects.update()
 
         # Set up the argument parsing
         self.__setup_parser()
         (opts, args) = self.__parser.parse_args(args)
-        os.chdir(opts.base_dir)
         setup_logging(opts.verbose)
 
         self.__opts = opts
         self.__args = args
 
-        if self.__opts.do_list:
-            for experiment in os.listdir(self.base_directory):
-                if experiment.startswith(self.title):
-                    print "EXP", experiment
-                    self.__do_list(self.__class__(experiment))
-            return None
-
         for key in kwargs:
             if not hasattr(opts, key):
                 raise AttributeError("No argument called %s" % key)
@@ -231,20 +270,26 @@ class Experiment(Type, InputParameter):
         # Set up the experiment
         self.before_experiment_run("output")
 
+    def execute_run(self):
+        # Goto the output directory
+        os.chdir(self.base_directory)
         try:
             self.run()
-        except:
+        except RuntimeError as e:
             # Clean up the tmp directory
             if self.suspend_on_error:
-                print "tmp-dir: %s" % self.tmp_directory.path
+                print(str(e))
+                print("tmp-dir: %s" % self.tmp_directory.path)
                 self.suspend_python()
             logging.error("Removing tmp directory")
             shutil.rmtree(self.tmp_directory.path)
-            raise
+            raise e
+        finally:
+            os.chdir(self.startup_directory)
 
-        # Tear down the experiment
+
+    def execute_teardown(self):
         self.after_experiment_run("output")
-
         return self.__experiment_instance
 
     @property
@@ -264,20 +309,6 @@ class Experiment(Type, InputParameter):
         os.kill(os.getpid(), signal.SIGSTOP)
 
 
-    def __do_list(self, experiment, indent = 0):
-        with open(os.path.join(experiment.base_directory, "metadata")) as fd:
-            content = fd.read()
-        d = eval(content)
-        content = experiment.__experiment_instance + "\n" + content
-        print "+%s%s" % ("-" * indent,
-                        content.strip().replace("\n", "\n|" + (" " * (indent+1))))
-        for dirname in d.values():
-            if type(dirname) != type(""):
-                continue
-            if os.path.exists(os.path.join(dirname, "metadata")) and \
-               os.path.realpath(dirname) != os.path.realpath(experiment.base_directory):
-                self.__do_list(Experiment(dirname), indent + 3)
-
     def before_experiment_run(self, parameter_type):
         # When experiment run as input, just run the normal input handlers
         if parameter_type == "input":
@@ -304,7 +335,8 @@ class Experiment(Type, InputParameter):
         self.subobjects.update()
 
         # Now set up the experiment tmp directory
-        self.__setup_tmp_directory()
+        self.tmp_directory = Directory(tempfile.mkdtemp())
+        self.subobjects["tmp_directory"] = self.tmp_directory
 
         for obj in self.inputs.values():
             obj.before_experiment_run("input")
@@ -320,15 +352,20 @@ class Experiment(Type, InputParameter):
         for name in self.inputs:
             metadata.update( self.inputs[name].inp_metadata() )
         m = hashlib.md5()
-        m.update("version %s" % str(self.version))
+        m.update(("version %s" % str(self.version)).encode())
         calc_metadata = self.filter_metadata(metadata)
         for key in sorted(calc_metadata.keys()):
-            m.update(key + " " + str(calc_metadata[key]))
+            m.update((key + " " + str(calc_metadata[key])).encode())
 
         self.__experiment_instance = "%s-%s" %(self.title, m.hexdigest())
-        self.base_directory = os.path.join(os.curdir, self.__experiment_instance)
+        if self.__opts.dummy_result:
+            base = self.tmp_directory.path
+        else:
+            base = self.__opts.base_dir
+        self.base_directory = os.path.join(base, self.__experiment_instance)
         self.base_directory = os.path.realpath(self.base_directory)
 
+
         if os.path.exists(self.base_directory):
             logging.info("Removing all files from existing output directory")
             for f in glob.glob(os.path.join(self.base_directory, '*')):
@@ -349,12 +386,19 @@ class Experiment(Type, InputParameter):
         metadata["experiment-version"] = self.version
         metadata["experiment-hash"]    = m.hexdigest()
 
-        fd = open(os.path.join(self.base_directory, "metadata"), "w+")
-        fd.write(pprint.pformat(metadata) + "\n")
-        fd.close()
+        with open(os.path.join(self.base_directory, "metadata"), "w") as fd:
+            json.dump(metadata, fd)
 
         self.__metadata = metadata
 
+
+    def symlink_name(self):
+        """If -s is given, this function returns the name of the symlink
+           object that is created in the base directory.
+
+        """
+        return self.title
+
     def after_experiment_run(self, parameter_type):
 
         if parameter_type == "output":
@@ -365,15 +409,18 @@ class Experiment(Type, InputParameter):
                 inp.after_experiment_run("input")
 
             self.__metadata["date-end"] = str(datetime.datetime.now())
-            fd = open(os.path.join(self.path, "metadata"), "w+")
-            fd.write(pprint.pformat(self.__metadata) + "\n")
-            fd.close()
+            with open(os.path.join(self.base_directory, "metadata"), "w") as fd:
+                json.dump(self.__metadata, fd)
 
             shutil.rmtree(self.tmp_directory.path)
 
-            # Create a Symlink to the newsest result set
-            if self.__opts.do_symlink:
-                link = os.path.join(self.__opts.base_dir, self.title)
+            # Create a Symlink to the newsest result set, if it is not
+            # already removed (--dummy)
+            if self.__opts.do_symlink and os.path.exists(self.base_directory):
+                link = os.path.join(self.base_directory,
+                                    "..",
+                                    self.symlink_name())
+                link = os.path.abspath(link)
                 if os.path.islink(link):
                     os.unlink(link)
 
@@ -393,20 +440,25 @@ class Experiment(Type, InputParameter):
     def inp_extract_cmdline_parser(self, opts, args):
         self.__experiment_instance = self.inp_parser_extract(opts, None)
         if not self.__experiment_instance:
-            print "Missing argument for %s" % self.title
+            print("Missing argument for %s" % self.title)
             raise ExperimentError
 
         # Resolve symlink relative to the current directory
-        self.__experiment_instance = os.path.realpath(self.__experiment_instance)
-        self.__experiment_instance = self.__experiment_instance[len(os.path.realpath(os.curdir))+1:]
+        path = os.path.realpath(self.__experiment_instance)
+        path = os.path.abspath(path)
+        self.__experiment_instance = os.path.basename(path)
 
-        self.base_directory = os.path.join(os.curdir, self.__experiment_instance)
-        self.base_directory = os.path.realpath(self.base_directory)
+        self.base_directory = path
+        assert os.path.exists(self.base_directory), \
+            "Base Directory does not exist"
+
+        self.__reinit__(path)
 
         for (name, outp) in self.outputs.items():
             del self.subobjects[name]
             self.subobjects[name] = outp
 
+
     def inp_metadata(self):
         return {self.name: self.__experiment_instance}
 
@@ -414,11 +466,11 @@ class Experiment(Type, InputParameter):
     def metadata(self):
         """Return the metadata as python dict. This works for
         experiments, which are running at the moment, and for already
-        run experiments by reading the /metadata file."""
+        finished experiments by reading the /metadata file."""
         if not self.__metadata:
             md_path = os.path.join(self.base_directory, "metadata")
             with open(md_path) as fd:
-                self.__metadata = eval(fd.read())
+                self.__metadata = json.load(fd)
         return self.__metadata
 
     @property
@@ -427,7 +479,7 @@ class Experiment(Type, InputParameter):
         return self.base_directory
 
     def run(self):
-        """This method is the hearth of every experiment and must be
+        """This method is the heart of every experiment and must be
         implemented by the user. It is called when the experiment is
         executed. Before all input parameters are parsed, the output
         directory is set up. Afterwards all temporary data is removed
@@ -470,8 +522,7 @@ class Experiment(Type, InputParameter):
              return inp
          elif outp != None:
              return outp
-         
+
          raise AttributeError("'%s' object has no attribute '%s'" %(\
              self.__class__.__name__,
              name))
-
